#!perl -w

# DBI->datasources
# DBI->connect
# DBI->connect_cached
# DBI::err
# DBI::errstr
# $dbh->{'Active'};

# All tests are stopped if we cannot make the first connect

use Test::More tests => 19;
use DBI;
use Sys::Hostname;
use strict;

# flush handle by setting autoflash flag $| to true
select((select(STDERR),$|=1)[0]);
$|=1;

# defaults for connection

my $dsn = $ENV{DBI_DSN};
my $user = "_SYSTEM";
my $passwd = "";
my $hostname = hostname;
my $dbname = "test";
my $dbpasswd = "";

# DSN_xxx values override defaults

if($dsn && $dsn =~ s/^(dbi):(\w*?)(?:\((.*?)\))?://i && lc($2) eq 'fb') {
	$dbpasswd = $1 if($3 && $3 =~ /\bfb_dbpasswd\b\s*=>\s*(\w+)/);
	$dbname = $1 if($dsn =~ /\bdbname\b\s*=\s*(\w+)/);
	$hostname = $1 if($dsn =~ /\bhost\b\s*=\s*([\.A-Za-z0-9_]+)/);
}
$user = $ENV{DBI_USER} if ($ENV{DBI_USER});
$passwd = $ENV{DBI_PASS} if ($ENV{DBI_PASS});

# FB_xxx values override DSN_xxx values

$hostname = $ENV{FB_HOST} if ($ENV{FB_HOST});
$dbname = $ENV{FB_DBNAME} if ($ENV{FB_DBNAME});
$dbpasswd = $ENV{FB_DBPASSWD} if ($ENV{FB_DBPASSWD});
$user = $ENV{FB_USER} if ($ENV{FB_USER});
$passwd = $ENV{FB_PASSWD} if ($ENV{FB_PASSWD});

my @datasources;

if($hostname eq hostname) {
	$hostname = 'localhost';
}

if( $hostname eq 'localhost') {
	@datasources = DBI->data_sources('FB');
}
else {
	@datasources = DBI->data_sources('FB', {fb_host => $hostname});
}

ok( @datasources, 'Check datasources');

my $testsource;

if (@datasources) {
	foreach my $datasource(@datasources) {
		if($datasource eq "dbi:FB:dbname=$dbname;host=$hostname") {
			$testsource = $datasource;
			last;
		}
	}
}

ok ($testsource, 'The database is available');

my $dbh;
eval {
	$dbh = DBI->connect($testsource, $user, $passwd,
											{fb_dbpasswd => $dbpasswd, RaiseError => 1, PrintError => 0, AutoCommit => 0});
};
if ($@) {
	if (! $DBI::errstr) {
		print STDOUT "Bail out! Could not connect: $@\n";
	}
	else {
		print STDOUT "Bail out! Could not connect: $DBI::errstr\n";
	}
	exit; # Force a hasty exit
}

pass('Established a connection to the database');

ok( $dbh->{'Active'}, 'Connection is active');
ok( $dbh->disconnect(), 'Disconnect from the database');
ok( !$dbh->{'Active'}, 'Connection is not active');

# Connect two times. From this point onward, do a simpler connection check
ok( $dbh = DBI->connect($testsource, $user, $passwd,
											{fb_dbpasswd => $dbpasswd, RaiseError => 0, PrintError => 0, AutoCommit => 0}),
		'Connected with first database handle');

my $dbh2;
ok( $dbh2 = DBI->connect($testsource, $user, $passwd,
											{fb_dbpasswd => $dbpasswd, RaiseError => 0, PrintError => 0, AutoCommit => 0}),
		'Connected with second database handle');
				
ok ( $dbh->disconnect(), 'Disconnect with first database handle');
ok ( $dbh2->disconnect(), 'Disconnect with second database handle');
ok ( $dbh2->disconnect(), 'Disconnect again with second database handle');

eval {
	$dbh = DBI->connect($testsource, '', '',
											{fb_dbpasswd => $dbpasswd, RaiseError => 1, PrintError => 0, AutoCommit => 0});
};

ok ($DBI::err == 1, 'Connection fails with FB_ERR_NO_CONNECTION');
ok ($@, 'Connect with unknown user fails');
ok (substr($DBI::errstr, 0, 35) eq 'Connection error 376. No such user.', 'Connection error is an unknown user');

ok( $dbh = DBI->connect_cached($testsource, $user, $passwd,
											{fb_dbpasswd => $dbpasswd, RaiseError => 0, PrintError => 0, AutoCommit => 0}),
		'Connected with first cached database handle');
		
ok( $dbh2 = DBI->connect_cached($testsource, $user, $passwd,
											{fb_dbpasswd => $dbpasswd, RaiseError => 0, PrintError => 0, AutoCommit => 0}),
		'Connected with second cached database handle');

ok ( $dbh == $dbh2, 'Compare cached database handles');

ok ( $dbh->disconnect(), 'Disconnect with first cached database handle');
ok ( $dbh2->disconnect(), 'Disconnect with second cached database handle');

END {
	my $pv = sprintf("%vd", $^V);
	diag 
		"\nProgram               Version\n".
		"Perl                  $pv ($^O)\n".
		"DBD::FB               $DBD::FB::VERSION\n".
		"DBI                   $DBI::VERSION\n"
}
