#!perl -w

# Test all handle attributes: database, statement, and generic ("any")

use Test::More tests => 106;
use DBI qw(:sql_types);
use Sys::Hostname;
use strict;
$|=1;

# defaults for connection

my $dsn = $ENV{DBI_DSN};
my $user = "_SYSTEM";
my $passwd = "";
my $hostname = hostname;
my $dbname = "test";
my $dbpasswd = "";

# DSN_xxx values override defaults

if($dsn && $dsn =~ s/^(dbi):(\w*?)(?:\((.*?)\))?://i && lc($2) eq 'fb') {
	$dbpasswd = $1 if($3 && $3 =~ /\bfb_dbpasswd\b\s*=>\s*(\w+)/);
	$dbname = $1 if($dsn =~ /\bdbname\b\s*=\s*(\w+)/);
	$hostname = $1 if($dsn =~ /\bhost\b\s*=\s*([\.A-Za-z0-9_]+)/);
}
$user = $ENV{DBI_USER} if ($ENV{DBI_USER});
$passwd = $ENV{DBI_PASS} if ($ENV{DBI_PASS});

# FB_xxx values override DSN_xxx values

$hostname = $ENV{FB_HOST} if ($ENV{FB_HOST});
$dbname = $ENV{FB_DBNAME} if ($ENV{FB_DBNAME});
$dbpasswd = $ENV{FB_DBPASSWD} if ($ENV{FB_DBPASSWD});
$user = $ENV{FB_USER} if ($ENV{FB_USER});
$passwd = $ENV{FB_PASSWD} if ($ENV{FB_PASSWD});

my $dbh = DBI->connect("dbi:FB:dbname=$dbname;host=$hostname", $user, $passwd,
							{fb_dbpasswd => $dbpasswd, RaiseError => 1, PrintError => 0, AutoCommit => 0});
				
ok( defined $dbh, "Connect to database for attribute testing");

my $attributes_tested = q{

d = database handle specific
s = statement handle specific
a = any type of handle (but we usually use database)

In order:

d Statement (must be the first one tested)
d CrazyDiamond (bogus)
d AutoCommit
d Driver
d Name
d RowCacheSize
d Username
d fb_auto_escape (should be private_dbdfb_auto_escape)

s NUM_OF_FIELDS, NUM_OF_PARAMS
s NAME, NAME_lc, NAME_uc, NAME_hash, NAME_lc_hash, NAME_uc_hash
s TYPE, PRECISION, SCALE, NULLABLE
s CursorName
s Database
s ParamValues
s ParamTypes
s RowsInCache

a Warn (inheritance test also)
a PrintWarn
a Active
a Kids
a ActiveKids
a CachedKids
a CompatMode
a PrintError
a RaiseError
a HandleError
a ShowErrorStatement (unsupported)
a TraceLevel
a FetchHashKeyName
a ChopBlanks
a LongReadLen
a LongTruncOk
a TaintIn
a TaintOut
a Taint
a Profile (not tested)

d InactiveDestroy (must be the last one tested)

};

my ($attrib,$SQL,$sth,$warning,$result,@result,$value,$testdata);

#
# Test of the database handle attribute "Statement"
#

$SQL = "values CURRENT_SCHEMA";
$sth = $dbh->prepare($SQL);
$sth->finish();

$attrib = $dbh->{Statement};
is( $attrib, $SQL, 'DB handle attribute "Statement" returns the last prepared query');

#
# Test of bogus database/statement handle attributes
#

## DBI switched from error to warning in 1.43
$warning="";
eval {
	local $SIG{__WARN__} = sub { $warning = shift; };
	$dbh->{CrazyDiamond}=1;
};
ok( (length $warning or $@), 'Error or warning when setting an invalid database handle attribute');

eval {
	$dbh->{private_dbdfb_CrazyDiamond}=1;
};
ok( !$@, 'Setting a private attribute on a database handle does not throw an error');

$sth = $dbh->prepare('values CURRENT_SCHEMA');

$warning="";
eval {
	local $SIG{__WARN__} = sub { $warning = shift; };
	$sth->{CrazyDiamond}=1;
};
ok( (length $warning or $@), 'Error or warning when setting an invalid statement handle attribute');

eval {
	$sth->{private_dbdfb_CrazyDiamond}=1;
};
ok( !$@, 'Setting a private attribute on a statement handle does not throw an error');

#
# Test of the database handle attribute "AutoCommit"
#

$dbh->do('DELETE FROM dbd_fb_test');
ok( $dbh->commit(), "Commit after deleting all rows from dbd_fb_test");

my $dbh2 = DBI->connect("dbi:FB:dbname=$dbname;host=$hostname", $user, $passwd,
							{fb_dbpasswd => $dbpasswd, RaiseError => 1, PrintError => 0});
							
ok( defined $dbh2, "Connect to database with second database handle, AutoCommit on");

is( $dbh2->{AutoCommit}, 1, 'AutoCommit on as default in second database handle');

$dbh->do('SET TRANSACTION ISOLATION LEVEL REPEATABLE READ, LOCKING DEFERRED, READ WRITE');
$dbh2->do('SET TRANSACTION ISOLATION LEVEL REPEATABLE READ, LOCKING DEFERRED, READ WRITE');

ok( $dbh->do("INSERT INTO dbd_fb_test (id, pname, val) VALUES (1, 'Coconut', 'Mango')"),
		'Insert a row into the database with first database handle');

my $rows = ($dbh2->selectrow_array(q{SELECT COUNT(*) FROM dbd_fb_test WHERE id = 1}))[0];
cmp_ok($rows, '==', 0, 'Second database handle cannot see insert from first');

ok( $dbh2->do("INSERT INTO dbd_fb_test (id, pname, val) VALUES (2, 'Grapefruit', 'Pomegranate')"),
		'Insert a row into the database with second database handle');

$rows = ($dbh->selectrow_array(q{SELECT COUNT(*) FROM dbd_fb_test WHERE id = 2}))[0];
cmp_ok($rows, '==', 1, 'First database handle can see insert from second');

ok( $dbh->commit, 'Commit transaction with first database handle');

$rows = ($dbh2->selectrow_array(q{SELECT COUNT(*) FROM dbd_fb_test WHERE id = 1}))[0];
cmp_ok($rows, '==', 1, 'Second database handle can see insert from first');

ok( $dbh2->disconnect(), 'Disconnect with second database handle');

$dbh->do('DELETE FROM dbd_fb_test');
$dbh->commit;

#
# Test of the database handle attribute "Driver"
#

$attrib = $dbh->{Driver}->{Name};
is( $attrib, 'FB', '$dbh->{Driver}{Name} returns correct value of "FB"');

#
# Test of the database handle attribute "Name"
#

{
	local $SIG{__WARN__} = sub {};
	$attrib = $dbh->{Name};
};

is($attrib, $dbname, 'DB handle attribute "Name" returns the same value as $dbname');

#
# Test of the database handle attribute "RowCacheSize"
# Not used.
#

$attrib = $dbh->{RowCacheSize};
ok( !defined $attrib, 'DB handle attribute "RowCacheSize" returns undef');
$dbh->{RowCacheSize} = 42;
$attrib = $dbh->{RowCacheSize};
ok( !defined $attrib, 'Setting DB handle attribute "RowCacheSize" has no effect');

#
# Test of the database handle attribute "Username"
#

if ($DBI::VERSION < 1.36) {
 SKIP: {
		skip 'DBI must be at least version 1.36 to test the DB handle attribute "Username"', 1;
	}
}
else {
	$attrib = $dbh->{Username};
	is( $attrib, $user, 'DB handle attribute "Username" returns the same value as $user');
}

#
# Test of the Frontbase specific database handle attribute "fb_auto_escape"
#

$value = $dbh->{fb_auto_escape};
is ($value, 1, qq{DB handle attribute "fb_auto_escape" defaults to on});

$SQL = "INSERT INTO dbd_fb_test(id,val) VALUES (?,?)";

$testdata = "Rose'apple";

$sth = $dbh->prepare($SQL);
$sth->bind_param(1, 1, SQL_INTEGER);
$sth->execute(10,$testdata);

$SQL = 'SELECT VAL FROM dbd_fb_test WHERE id = 10';

@result = $dbh->selectrow_array($SQL);
is($result[0], $testdata, 'Quote was stored correctly with fb_auto_escape on');

$dbh->rollback;

$dbh->{fb_auto_escape} = 0;
$value = $dbh->{fb_auto_escape};
is ($value, 0, qq{DB handle attribute "fb_auto_escape" is off});

$SQL = "INSERT INTO dbd_fb_test(id,val) VALUES (?,?)";

$testdata = "Rose''apple";

$sth->execute(10,$testdata);

$SQL = 'SELECT VAL FROM dbd_fb_test WHERE id = 10';

$testdata = "Rose'apple";

@result = $dbh->selectrow_array($SQL);
is($result[0], $testdata, 'Quote was stored correctly with fb_auto_escape off');

$dbh->rollback;

$dbh->{fb_auto_escape} = 1;

#
# Use the handle attribute "Warn" to check inheritance
# This is also tested from other tests using $SIG{__WARN__}.
#

undef $sth;

ok( $dbh->{Warn}, 'Attribute "Warn" attribute set on by default');

$SQL = "values 123";
$sth = $dbh->prepare($SQL);
$sth->finish();
ok( $sth->{Warn}, 'Statement handle inherits the "Warn" attribute');

$dbh->{Warn} = 0;
ok( ! $dbh->{Warn}, 'Turn off the "Warn" attribute in the database handle');

#
# Test of the "PrintWarn" database handle attribute
# The driver does not return warnings.
#

$value = $dbh->{PrintWarn};
is ($value, 1, qq{DB handle attribute "PrintWarn" defaults to on});

{

local $SIG{__WARN__} = sub { $warning = shift; };

$warning = q{};
eval {
	$dbh->do("DELETE FROM dbd_fb_test");
};
ok (!$@, qq{DB handle attribute "PrintWarn" works when on});
#$warning = 'Completion condition 250. No data.';
like($warning, qr{No data}, qq{DB handle attribute "PrintWarn" shows warnings when on});

$dbh->{PrintWarn}=0;
$warning = q{};
eval {
	$dbh->do("DELETE FROM dbd_fb_test");
};
ok (!$@, qq{DB handle attribute "PrintWarn" works when on});
is($warning, q{}, qq{DB handle attribute "PrintWarn" doesn't return warnings when off});

$dbh->{PrintWarn}=1;

}

#
# Test of the the following statement handle attributes:
# NUM_OF_FIELDS, NUM_OF_PARAMS
# NAME, NAME_lc, NAME_uc, NAME_hash, NAME_lc_hash, NAME_uc_hash
# TYPE, PRECISION, SCALE, NULLABLE
#

$sth = $dbh->prepare('SELECT 123 AS "Sheep", CAST(id AS FLOAT) AS "id", CAST(12.12 AS NUMERIC(15,2)) AS "Dec" FROM dbd_fb_test WHERE id=?');
$sth->execute(12);
$attrib = $sth->{'NUM_OF_FIELDS'};
is( $attrib, '3', 'Statement handle attribute "NUM_OF_FIELDS" works correctly for SELECT');
$attrib = $sth->{'NUM_OF_PARAMS'};
is( $attrib, '1', 'Statement handle attribute "NUM_OF_PARAMS" works correctly with one placeholder');
$attrib = $sth->{NAME};
my $colnames = ['Sheep', 'id', 'Dec'];
is_deeply( $attrib, $colnames, 'Statement handle attribute "NAME" works correctly');
$attrib = $sth->{NAME_lc};
$colnames = ['sheep', 'id', 'dec'];
is_deeply( $attrib, $colnames, 'Statement handle attribute "NAME_lc" works correctly');
$attrib = $sth->{NAME_uc};
$colnames = ['SHEEP', 'ID', 'DEC'];
is_deeply( $attrib, $colnames, 'Statement handle attribute "NAME_uc" works correctly');
$attrib = $sth->{'NAME_hash'};
$colnames = {'Sheep' => 0, id => 1, Dec => 2};
is_deeply( $attrib, $colnames, 'Statement handle attribute "NAME_hash" works correctly');
$attrib = $sth->{'NAME_lc_hash'};
$colnames = {sheep => 0, id => 1, dec => 2};
is_deeply( $attrib, $colnames, 'Statement handle attribute "NAME_lc_hash" works correctly');
$attrib = $sth->{NAME_uc_hash};
$colnames = {SHEEP => 0, ID => 1, DEC => 2};
is_deeply( $attrib, $colnames, 'Statement handle attribute "NAME_uc_hash" works correctly');
  
$attrib = $sth->{TYPE};
$colnames = [5, 6, 2];
is_deeply( $attrib, $colnames, 'Statement handle attribute "TYPE" works correctly');

$attrib = $sth->{PRECISION};
$colnames = [5, 15, 15];
is_deeply( $attrib, $colnames, 'Statement handle attribute "PRECISION" works correctly');

# Not supported. Should return the same as the ODBC driver SQLDescribeCol(DECIMAL_DIGITS)

$attrib = $sth->{SCALE};
$colnames = [undef, undef, 2];
is_deeply( $attrib, $colnames, 'Statement handle attribute "SCALE" works correctly');

# Not supported. Should return the same as ODBC.

$sth = $dbh->prepare('SELECT id, lii FROM dbd_fb_test WHERE id=?');
$sth->execute(12);

$attrib = $sth->{NULLABLE};
$colnames = [0,1];
is_deeply( $attrib, $colnames, 'Statement handle attribute "NULLABLE" works correctly');

$sth->finish();

$sth = $dbh->prepare("DELETE FROM dbd_fb_test WHERE id=0");
$sth->execute();
$attrib = $sth->{'NUM_OF_FIELDS'};

# maybe this should return 0 instead of undef

is( $attrib, undef, 'Statement handle attribute "NUM_OF_FIELDS" works correctly for DELETE');
$attrib = $sth->{'NUM_OF_PARAMS'};
is( $attrib, '0', 'Statement handle attribute "NUM_OF_PARAMS" works correctly with no placeholder');
$attrib = $sth->{NAME};
$colnames = [];
is_deeply( $attrib, $colnames, 'Statement handle attribute "NAME" works correctly for DELETE');

$sth->finish();

#
# Test of the statement handle attribute "CursorName".
# Not supported.
#

$attrib = $sth->{CursorName};
is( $attrib, undef, 'Statement handle attribute "CursorName" returns undef');

#
# Test of the statement handle attribute "Database"
#

$attrib = $sth->{Database};
is( $attrib, $dbh, 'Statement handle attribute "Database" matches the database handle');

#
# Test of the statement handle attribute "ParamValues"
# Not supported
#

$sth = $dbh->prepare("SELECT id FROM dbd_pg_test WHERE id=? AND val IS ? AND pname=?");
$sth->bind_param(1, 99);
$sth->bind_param(3, 'Sparky');
$attrib = $sth->{ParamValues};
is( $attrib, undef, 'Statement handle attribute "ParamValues" returns undef');

#
# Test of the statement handle attribute "ParamTypes"
# Not supported
#

if ($DBI::VERSION < 1.49) {
 SKIP: {
		skip 'DBI must be at least version 1.49 to test the DB handle attribute "ParamTypes"', 2;
	}
}
else {
	$sth = $dbh->prepare("SELECT id FROM dbd_fb_test WHERE id=? AND val=? AND lii=?");
	$sth->bind_param(1, 1, SQL_INTEGER);
	$sth->bind_param(2, 'TMW', SQL_VARCHAR);
	$attrib = $sth->{ParamTypes};
	is_deeply( $attrib, undef, 'Statement handle attribute "ParamTypes" returns undef');
}

#
# Test of the statement handle attribute "RowsInCache"
# Not supported
#

$attrib = $sth->{RowsInCache};
is( $attrib, undef, 'Statement handle attribute "RowsInCache" returns undef');

#
# Test of the statement handle attribute "fb_size"
#

$SQL = 'SELECT * FROM dbd_fb_test';
$sth = $dbh->prepare($SQL);
$sth->execute();
$result = $sth->{fb_size};
my $expected = [qw(4 4 20 255 8 5 19 1 17)];
is_deeply( $result, $expected, 'Statement handle attribute "pg_size" works');

#
# Test of the statement handle attribute "fb_type"
#

$sth->execute();
$result = $sth->{fb_type};
$expected = [qw(integer integer varchar varchar float character timestamp boolean bit)];
is_deeply( $result, $expected, 'Statement handle attribute "fb_type" works');
$sth->finish();

#
# Test of the handle attribute "Active"
#

$attrib = $dbh->{Active};
is( $attrib, 1, 'Database handle attribute "Active" is true while connected');

$sth = $dbh->prepare("VALUES 123 UNION VALUES 456");
$attrib = $sth->{Active};
is($attrib, '', 'Statement handle attribute "Active" is false before SELECT');
$sth->execute();
$attrib = $sth->{Active};
is($attrib, 1, 'Statement handle attribute "Active" is true after SELECT');
my $row = $sth->fetchrow_arrayref();
$attrib = $sth->{Active};
is($attrib, 1, 'Statement handle attribute "Active" is true when rows remaining');
$sth->finish();
$attrib = $sth->{Active};
is($attrib, '', 'Statement handle attribute "Active" is false after finish called');

#
# Test of the handle attribute "Kids"
#

$attrib = $dbh->{Kids};
is( $attrib, 1, 'Database handle attribute "Kids" is set properly');
my $sth2 = $dbh->prepare("VALUES 234");
$attrib = $dbh->{Kids};
is( $attrib, 2, 'Database handle attribute "Kids" works');
$attrib = $sth2->{Kids};
is( $attrib, 0, 'Statement handle attribute "Kids" is zero');

#
# Test of the handle attribute "ActiveKids"
#

$attrib = $dbh->{ActiveKids};
is( $attrib, 0, 'Database handle attribute "ActiveKids" is set properly');
$sth2 = $dbh->prepare("VALUES 234");
$sth2->execute();
$attrib = $dbh->{ActiveKids};
is( $attrib, 1, 'Database handle attribute "ActiveKids" works');
$attrib = $sth2->{ActiveKids};
is( $attrib, 0, 'Statement handle attribute "ActiveKids" is zero');

#
# Test of the handle attribute "CachedKids"
#

$attrib = $dbh->{CachedKids};
ok( !$attrib, 'Database handle attribute "CachedKids" is set properly');

#
# Test of the handle attribute "CompatMode"
#

$attrib = $dbh->{CompatMode};
ok( !$attrib, 'Database handle attribute "CompatMode" is set properly');

#
# Test of the handle attribute PrintError
#

$attrib = $dbh->{PrintError};
is( $attrib, '', 'Database handle attribute "PrintError" is set properly');

$SQL = "Testing the DBD::FB modules error handling -?-";
	{
		$warning = '';
		local $SIG{__WARN__} = sub { $warning = shift; };
		$dbh->{RaiseError} = 0;
		
		$dbh->{PrintError} = 1;
		$sth = $dbh->prepare($SQL);
		$sth->execute();
		ok( $warning, 'Warning thrown when database handle attribute "PrintError" is on');

		undef $warning;
		$dbh->{PrintError} = 0;
		$sth = $dbh->prepare($SQL);
		$sth->execute();
		ok( !$warning, 'No warning thrown when database handle attribute "PrintError" is off');
	}

#
# Test of the handle attribute RaiseError
#
	
eval {
	$sth = $dbh->prepare($SQL);
	$sth->execute();
};
ok (!$@, 'No error produced when database handle attribute "RaiseError" is off');

$dbh->{RaiseError} = 1;
eval {
	$sth = $dbh->prepare($SQL);
	$sth->execute();
};
ok ($@, 'Error produced when database handle attribute "RaiseError" is off');

#
# Test of the handle attribute HandleError
#

$attrib = $dbh->{HandleError};
ok( !$attrib, 'Database handle attribute "HandleError" is set properly');

undef $warning;
$dbh->{HandleError} = sub { $warning = shift; };
$sth = $dbh->prepare($SQL);
$sth->execute();
ok( $warning, 'Database handle attribute "HandleError" works');
# Test changing values
undef $warning;
$dbh->{HandleError} = sub { $_[0] = "FrontBase $_[0]"; 0; };
eval {
	$sth = $dbh->prepare($SQL);
	$sth->execute();
};

like($@, qr/^FrontBase/, 'Database handle attribute "HandleError" modifies error messages');
$dbh->{HandleError}= undef;
$dbh->rollback();

#
# Test of the handle attribute ShowErrorStatement
#

{
	$warning = '';
	local $SIG{__WARN__} = sub { $warning = shift; };
	$dbh->{ShowErrorStatement} = 1;
	$dbh->{RaiseError} = 0;
	$dbh->{PrintError} = 1;
	$sth = $dbh->prepare($SQL);
	$sth->execute();
	like($warning, qr/$SQL/, 'Database handle attribute "ShowErrorStatement" modifies error messages');

	undef $warning;
	$dbh->{ShowErrorStatement} = 0;
	$sth = $dbh->prepare($SQL);
	$sth->execute();
	unlike($warning, qr/$SQL/, 'Database handle attribute "ShowErrorStatement" modifies error messages');
}

#
# Test of the handle attribute TraceLevel
#

$attrib = $dbh->{TraceLevel};
like($attrib, qr/^\d$/, qq{Database handle attribute "TraceLevel" returns a number ($attrib)});

#
# Test of the handle attribute FetchHashKeyName
#

ok( $dbh->do("INSERT INTO dbd_fb_test (id) VALUES (1)"), 'Insert a row into the database');

# The default is mixed case ("NAME");
$attrib = $dbh->{FetchHashKeyName};
is( $attrib, 'NAME', 'Database handle attribute "FetchHashKeyName" is set properly');

$SQL = qq{SELECT "CaseTest" FROM dbd_fb_test};
$sth = $dbh->prepare($SQL);
$sth->execute();
my ($colname) = keys %{$sth->fetchrow_hashref()};
is( $colname, 'CaseTest', 'Database handle attribute "FetchHashKeyName" works with the default value of NAME');
$sth->finish();

$dbh->{FetchHashKeyName} = "NAME_lc";
$attrib = $dbh->{FetchHashKeyName};
is( $attrib, 'NAME_lc', 'Database handle attribute "FetchHashKeyName" can be changed');

$sth = $dbh->prepare($SQL);
$sth->execute();
($colname) = keys %{$sth->fetchrow_hashref()};
is( $colname, 'casetest', 'Database handle attribute "FetchHashKeyName" works with a value of NAME_lc');
$sth->finish();

$dbh->{FetchHashKeyName} = "NAME_uc";
$sth = $dbh->prepare($SQL);
$sth->execute();
($colname) = keys %{$sth->fetchrow_hashref()};
is( $colname, 'CASETEST', 'Database handle attribute "FetchHashKeyName" works with a value of NAME_uc');
$sth->finish();
$dbh->{FetchHashKeyName} = "NAME";

#
# Test of the handle attribute ChopBlanks
# Not working correctly. If not supported the attribute should return undef.
#

$attrib = $dbh->{ChopBlanks};
ok( !$attrib, 'Database handle attribute "ChopBlanks" is set properly');

$dbh->do("DELETE FROM dbd_fb_test");
$dbh->do(q{INSERT INTO dbd_fb_test (id, fixed, val) VALUES (3, ' Fig ', ' Raspberry ')});

$dbh->{ChopBlanks} = 0;
my ($val) = $dbh->selectall_arrayref(q{SELECT fixed FROM dbd_fb_test WHERE id = 3})->[0][0];
is( $val, ' Fig ', 'Database handle attribute "ChopBlanks" = 0 returns correct value for fixed-length column');
($val) = $dbh->selectrow_array(q{SELECT val FROM dbd_fb_test WHERE id = 3});
is( $val, ' Raspberry ', 'Database handle attribute "ChopBlanks" = 0 returns correct value for variable-length column');

$dbh->{ChopBlanks}=1;

($val) = $dbh->selectall_arrayref(q{SELECT fixed FROM dbd_fb_test WHERE id = 3})->[0][0];
is( $val, ' Fig', 'Database handle attribute "ChopBlanks" = 1 returns correct value for fixed-length column');

($val) = $dbh->selectrow_array(q{SELECT val FROM dbd_fb_test WHERE id = 3});
is( $val, ' Raspberry ', 'Database handle attribute "ChopBlanks" = 1 returns correct value for variable-length column');
$dbh->do("DELETE from dbd_fb_test");

#
# Test of the handle attribute LongReadLen
#

$attrib = $dbh->{LongReadLen};
ok( $attrib, 'Handle attribute "LongReadLen" has been set properly');

#
# Test of the handle attribute LongTruncOk
#

$attrib = $dbh->{LongTruncOk};
ok( !$attrib, 'Handle attribute "LongTruncOk" has been set properly');

#
# Test of the handle attribute TaintIn
#

$attrib = $dbh->{TaintIn};
is( $attrib, '', 'Handle attribute "TaintIn" has been set properly');

#
# Test of the handle attribute TaintOut
#

$attrib = $dbh->{TaintOut};
is( $attrib, '', 'Handle attribute "TaintOut" has been set properly');

#
# Test of the handle attribute Taint
#
$attrib = $dbh->{Taint};
is( $attrib, '', 'Handle attribute "Taint" has been set properly');

$dbh->{Taint}=1;

$attrib = $dbh->{Taint};
is( $attrib, 1, 'The value of handle attribute "Taint" can be changed');
$attrib = $dbh->{TaintIn};
is( $attrib, 1, 'Changing handle attribute "Taint" changes "TaintIn"');
$attrib = $dbh->{TaintOut};
is( $attrib, 1, 'Changing handle attribute "Taint" changes "TaintOut"');

#
# Not tested: handle attribute Profile
#

#
# Test of the database handle attribute InactiveDestroy
# This one must be the last test performed!
# Test doesn't currently work so SKIP.
#

$attrib = $dbh->{InactiveDestroy};
ok( !$attrib, 'Database handle attribute "InactiveDestroy" is set properly');

# Disconnect in preparation for the fork tests
ok( $dbh->disconnect(), 'Disconnect from database');
$attrib = $dbh->{Active};
is( $attrib, '', 'Database handle attribute "Active" is false after disconnect');


if ($^O =~ /MSWin/) {
 SKIP: {
		skip 'Cannot test database handle "InactiveDestroy" on a non-forking system', 5;
	}
}
else {
	require Test::Simple;
	if ($Test::Simple::VERSION < 0.47) {
	SKIP: {
			skip 'Test::Simple version 0.47 or better required for testing of attribute "InactiveDestroy"', 5;
		}
	}
	else {

		# Test of forking. Hang on to your hats
		SKIP: {
			skip 'Test of "InactiveDestroy" needs some work', 5;
			my $answer = 42;
			$SQL = "VALUES $answer";

			for my $destroy (0,1) {

				$dbh = DBI->connect("dbi:FB:dbname=$dbname;host=$hostname", $user, $passwd,
								{fb_dbpasswd => $dbpasswd, RaiseError => 0, PrintError => 0, AutoCommit => 1});

				$sth = $dbh->prepare($SQL);
				$sth->execute();
				$sth->finish();

				# Desired flow: parent test, child test, child kill, parent test

				if (fork) {
					$sth->execute();
					my $val = $sth->fetchall_arrayref()->[0][0];
					is( $val, $answer, qq{Parent in fork test is working properly ("InactiveDestroy" = $destroy)});
					# Let the child exit
					select(undef,undef,undef,0.3);
				}
				else { # Child
					$dbh->{InactiveDestroy} = $destroy;
					select(undef,undef,undef,0.1); # Age before beauty
					exit; ## Calls disconnect via DESTROY unless InactiveDestroy set
				}

				if ($destroy) {
					# The database handle should still be active
					ok ( $dbh->ping(), qq{Ping works after the child has exited ("InactiveDestroy" = $destroy)});
					## The statement handle should still be usable
					$sth->execute();
					my $val = $sth->fetchall_arrayref()->[0][0];
					is ($val, $answer, qq{Statement handle works after forking});
				}
				else {
					# The database handle should be dead
					ok ( !$dbh->ping(), qq{Ping fails after the child has exited ("InactiveDestroy" = $destroy)});
				}
			}
		}
	}
}

