#!perl -w

# Test of the database handle methods
# data_sources : needs updating when statement method is written
# do

use Test::More tests => 105;
use DBI qw(:sql_types);
use DBI::Const::GetInfoType;
use Sys::Hostname;
use strict;
$|=1;

# defaults for connection

my $dsn = $ENV{DBI_DSN};
my $user = "_SYSTEM";
my $passwd = "";
my $hostname = hostname;
my $dbname = "test";
my $dbpasswd = "";

# DSN_xxx values override defaults

if($dsn && $dsn =~ s/^(dbi):(\w*?)(?:\((.*?)\))?://i && lc($2) eq 'fb') {
	$dbpasswd = $1 if($3 && $3 =~ /\bfb_dbpasswd\b\s*=>\s*(\w+)/);
	$dbname = $1 if($dsn =~ /\bdbname\b\s*=\s*(\w+)/);
	$hostname = $1 if($dsn =~ /\bhost\b\s*=\s*([\.A-Za-z0-9_]+)/);
}
$user = $ENV{DBI_USER} if ($ENV{DBI_USER});
$passwd = $ENV{DBI_PASS} if ($ENV{DBI_PASS});

# FB_xxx values override DSN_xxx values

$hostname = $ENV{FB_HOST} if ($ENV{FB_HOST});
$dbname = $ENV{FB_DBNAME} if ($ENV{FB_DBNAME});
$dbpasswd = $ENV{FB_DBPASSWD} if ($ENV{FB_DBPASSWD});
$user = $ENV{FB_USER} if ($ENV{FB_USER});
$passwd = $ENV{FB_PASSWD} if ($ENV{FB_PASSWD});

my $dbh = DBI->connect("dbi:FB:dbname=$dbname;host=$hostname", $user, $passwd,
							{fb_dbpasswd => $dbpasswd, RaiseError => 1, PrintError => 0, AutoCommit => 0});
				
ok( defined $dbh, "Connect to database for database handle method testing");

#
# Test data_sources database handle method
#
# this currently only returns databases from either localhost or fb_host
# when no attribute is specified it should return the connected hosts databases.

my @datasources;

if($hostname eq hostname) {
	$hostname = 'localhost';
}

if( $hostname eq 'localhost') {
	@datasources = $dbh->data_sources();
}
else {
	@datasources = $dbh->data_sources({fb_host => $hostname});
}

ok( @datasources, 'Check datasources');

my $testsource;

if (@datasources) {
	foreach my $datasource(@datasources) {
		if($datasource eq "dbi:FB:dbname=$dbname;host=$hostname") {
			$testsource = $datasource;
			last;
		}
	}
}

ok (defined $testsource, 'The database is available');

my ($SQL, $sth, $result, @result, $expected, $warning, $rows, $catalog);

# store catalog for meta-data tests

$result = $dbh->selectcol_arrayref("VALUES CURRENT_CATALOG");
$catalog = (@$result)[0];

# Populate the testing table for later use

$SQL = "INSERT INTO dbd_fb_test(id,val) VALUES (?,?)";

$sth = $dbh->prepare($SQL);
$sth->bind_param(1, 1, SQL_INTEGER);
$sth->execute(10,'Roseapple');
$sth->execute(11,'Pineapple');
$sth->execute(12,'Kiwi');

#
# Test of the "last_insert_id" database handle method
# Not currently supported by FrontBase
#

if ($DBI::VERSION <= 1.42) {
 SKIP: {
		skip 'DBI must be at least version 1.43 to completely test database handle method "last_insert_id"', 1;
	}
}
elsif (!$dbh->can('last_insert_id')) {
	ok(!defined $dbh->last_insert_id(undef, undef, undef, undef), 'last_insert_id is not defined');
	diag 'Driver does not support "last_insert_id"';
}
else
{
	ok($dbh->last_insert_id(undef,undef,undef,undef) == 3, 'Last row was INDEX 3');	
}

#
# Test of the "selectrow_array" database handle method
#

$SQL = "SELECT id FROM dbd_fb_test ORDER BY id";
@result = $dbh->selectrow_array($SQL);
$expected = [10];
is_deeply(\@result, $expected, 'DB handle method "selectrow_array" works');

#
# Test of the "selectrow_arrayref" database handle method
#

$result = $dbh->selectrow_arrayref($SQL);
is_deeply($result, $expected, 'DB handle method "selectrow_arrayref" works');

$sth = $dbh->prepare($SQL);
$result = $dbh->selectrow_arrayref($sth);
is_deeply($result, $expected, 'DB handle method "selectrow_arrayref" works with a prepared statement handle');

#
# Test of the "selectrow_hashref" database handle method
#

$result = $dbh->selectrow_hashref($SQL);
$expected = {id => 10};
is_deeply($result, $expected, 'DB handle method "selectrow_hashref" works');

$sth = $dbh->prepare($SQL);
$result = $dbh->selectrow_hashref($sth);
is_deeply($result, $expected, 'DB handle method "selectrow_hashref" works with a prepared statement handle');

#
# Test of the "selectall_arrayref" database handle method
#

$result = $dbh->selectall_arrayref($SQL);
$expected = [[10],[11],[12]];
is_deeply($result, $expected, 'DB handle method "selectall_arrayref" works');

$sth = $dbh->prepare($SQL);
$result = $dbh->selectall_arrayref($sth);
is_deeply($result, $expected, 'DB handle method "selectall_arrayref" works with a prepared statement handle');

$result = $dbh->selectall_arrayref($SQL, {MaxRows => 2});
$expected = [[10],[11]];
is_deeply($result, $expected, 'DB handle method "selectall_arrayref" works with the MaxRows attribute');

$SQL = "SELECT id, val FROM dbd_fb_test ORDER BY id";
$result = $dbh->selectall_arrayref($SQL, {Slice => [1]});
$expected = [['Roseapple'],['Pineapple'],['Kiwi']];
is_deeply($result, $expected, 'DB handle method "selectall_arrayref" works with the Slice attribute');

#
# Test of the "selectall_hashref" database handle method
#

$result = $dbh->selectall_hashref($SQL,'id');
$expected = {10=>{id =>10,val=>'Roseapple'},11=>{id=>11,val=>'Pineapple'},12=>{id=>12,val=>'Kiwi'}};
is_deeply($result, $expected, 'DB handle method "selectall_hashref" works');

$sth = $dbh->prepare($SQL);
$result = $dbh->selectall_hashref($sth,'id');
is_deeply($result, $expected, 'DB handle method "selectall_hashref" works with a prepared statement handle');

#
# Test of the "selectcol_arrayref" database handle method
#

$result = $dbh->selectcol_arrayref($SQL);
$expected = [10,11,12];
is_deeply($result, $expected, 'DB handle method "selectcol_arrayref" works');

$result = $dbh->selectcol_arrayref($sth);
is_deeply($result, $expected, 'DB handle method "selectcol_arrayref" works with a prepared statement handle');

$result = $dbh->selectcol_arrayref($SQL, {Columns=>[2,1]});
$expected = ['Roseapple',10,'Pineapple',11,'Kiwi',12];
is_deeply($result, $expected, 'DB handle method "selectcol_arrayref" works with the Columns attribute');

if ($DBI::VERSION < 1.36) {
 SKIP: {
		skip 'DBI must be at least version 1.36 to test "selectcol_arrayref" with "MaxRows"', 1;
	}
}
else {
	$result = $dbh->selectcol_arrayref($SQL, {Columns=>[2], MaxRows => 1});
	$expected = ['Roseapple'];
	is_deeply($result, $expected, 'DB handle method "selectcol_arrayref" works with the MaxRows attribute');
}

#
# Test of the "commit" and "rollback" database handle methods.
# Also tests AutoCommit database handle attribute.
#

$sth->finish;
$dbh->disconnect;
$dbh = DBI->connect("dbi:FB:dbname=$dbname;host=$hostname", $user, $passwd,
							{fb_dbpasswd => $dbpasswd, RaiseError => 1, PrintError => 0});
							
my $dbh2 = DBI->connect("dbi:FB:dbname=$dbname;host=$hostname", $user, $passwd,
							{fb_dbpasswd => $dbpasswd, RaiseError => 1, PrintError => 0, AutoCommit => 0});

ok( defined $dbh, 'Reconnect for autocommit test');
ok( defined $dbh2, 'Second db handle for autocommit test');
is( $dbh->{AutoCommit}, 1, 'AutoCommit on as default');

{
	local $SIG{__WARN__} = sub { $warning = shift; };
	
	eval {
		$sth = $dbh->prepare('DROP TABLE dbd_fb_auto CASCADE');
		$sth->execute;
	};
	
	eval {
		$sth = $dbh->prepare('DROP TABLE dbd_fb_auto2 CASCADE');
		$sth->execute;
	};
	
	
	$sth = $dbh->prepare('CREATE TABLE dbd_fb_auto(c0 int)');
	$sth->execute;
		
	eval {
		$sth = $dbh->prepare('values current_bun');
		$sth->execute;
	};
	
	@result = $dbh2->tables;
	my $name;
	for(@result) {
		if($_ eq 'dbd_fb_auto') {
			$name = $_;
			last;
		}
	}
	
	$dbh2->rollback;
	
	is($name, 'dbd_fb_auto', 'CREATE was auto committed');
	
	$warning="";
	$dbh->commit();
	ok( length $warning, 'DB handle method "commit" gives a warning when AutoCommit is on');
	$warning="";
	$dbh->rollback();
	ok( length $warning, 'DB handle method "rollback" gives a warning when AutoCommit is on');

	$dbh->{AutoCommit}=0;
	
	is( $dbh->{AutoCommit}, '', 'AutoCommit turned off');
	
	$sth = $dbh->prepare('CREATE TABLE dbd_fb_auto2(c0 int)');
	$sth->execute;
	$dbh->{AutoCommit}=1;
	
	@result = $dbh2->tables;
	undef $name;
	for(@result) {
		if($_ eq 'dbd_fb_auto2') {
			$name = $_;
			last;
		}
	}
	is($name, 'dbd_fb_auto2', 'Turning on autocommit did a commit');
	
	$dbh2->rollback;
	$dbh->{AutoCommit}=0;
	
	$warning="";
	$sth = $dbh->prepare('DROP TABLE dbd_fb_auto CASCADE');
	$sth->execute;
	$dbh->rollback();
	ok( ! length $warning, 'DB handle method "rollback" gives no warning when AutoCommit is off');
	
	@result = $dbh2->tables;
	undef $name;
	for(@result) {
		if($_ eq 'dbd_fb_auto') {
			$name = $_;
			last;
		}
	}
	$dbh2->rollback;
	is($name, 'dbd_fb_auto', 'ROLLBACK did not drop table');

	$warning="";
	$sth = $dbh->prepare('DROP TABLE dbd_fb_auto CASCADE');
	$sth->execute;
	$dbh->commit();
	ok( ! length $warning, 'DB handle method "commit" gives no warning when AutoCommit is off');
	
	@result = $dbh2->tables;
	undef $name;
	for(@result) {
		if($_ eq 'dbd_fb_auto') {
			$name = $_;
			last;
		}
	}
	$dbh2->rollback;
	isnt($name, 'dbd_fb_auto', 'COMMIT did drop table');
	
	$warning="";
	$sth = $dbh->prepare('DROP TABLE dbd_fb_auto2 CASCADE');
	$sth->execute;
	$dbh->commit();
	ok( ! length $warning, 'DB handle method "commit" gives no warning when AutoCommit is off');
	
	@result = $dbh2->tables;
	undef $name;
	for(@result) {
		if($_ eq 'dbd_fb_auto2') {
			$name = $_;
			last;
		}
	}
	$dbh2->disconnect;
	isnt($name, 'dbd_fb_auto2', 'COMMIT did drop table');
	
	ok( $dbh->commit, 'DB handle method "commit" returns true');
	ok( $dbh->rollback, 'DB handle method "rollback" returns true');
}

#
# Test of the "begin_work" database handle method
# Note. FrontBase does not support an explicit BEGIN
#

$dbh->{AutoCommit}=0;
eval {
	$dbh->begin_work();
};
ok( $@, 'DB handle method "begin_work" gives a warning when AutoCommit is on');

$dbh->{AutoCommit}=1;
eval {
	$dbh->begin_work();
};
ok( !$@, 'DB handle method "begin_work" gives no warning when AutoCommit is off');
ok( !$dbh->{AutoCommit}, 'DB handle method "begin_work" sets AutoCommit to off');
$dbh->commit();
ok( $dbh->{AutoCommit}, 'DB handle method "commit" after "begin_work" sets AutoCommit to on');

$dbh->{AutoCommit}=1;
eval {
	$dbh->begin_work();
};
ok( !$@, 'DB handle method "begin_work" gives no warning when AutoCommit is off');
ok( !$dbh->{AutoCommit}, 'DB handle method "begin_work" sets AutoCommit to off');
$dbh->rollback();
ok( $dbh->{AutoCommit}, 'DB handle method "rollback" after "begin_work" sets AutoCommit to on');

$dbh->{AutoCommit}=0;

eval {
  $dbh->get_info();
};
ok ($@, 'DB handle method "get_info" with no arguments gives an error');

#
# Test of the "get_info" database handle method
#

my %get_info = (
  SQL_DRIVER_NAME            => 6,
  SQL_DRIVER_VER             => 7,
  SQL_DATABASE_NAME			 => 16,
  SQL_DBMS_NAME              => 17,
  SQL_DBMS_VERSION           => 18,
  SQL_IDENTIFIER_QUOTE_CHAR  => 29,
  SQL_CATALOG_NAME_SEPARATOR => 41,
  SQL_USER_NAME				 => 47,
  SQL_CATALOG_LOCATION       => 114,
);

if (!$dbh->can('get_info')) {
	ok(!defined $dbh->get_info(17), 'get_info is not defined');
	diag 'Driver does not support "get_info"';
	SKIP: {
		skip 'Driver does not support "get_info"', 26;
	}
}
else {
	for (keys %get_info) {  
		my $back = $dbh->get_info($GetInfoType{$_});
		ok( defined $back, qq{DB handle method "get_info" works with a value of "$_"});
		my $forth = $dbh->get_info($get_info{$_});
		ok( defined $forth, qq{DB handle method "get_info" works with a value of "$get_info{$_}"});
		diag "$_ = $forth";
		is( $back, $forth, qq{DB handle method "get_info" returned matching values});
	}
}

#
# Test of the "table_info" database handle method
# Note. This doesn't return REMARKS and does not support any parameters.
# All objects are returned from all schemas except the INFORMATION_SCHEMA (system tables/views).
#

# Check required minimum fields
# Note. This should return a REMARKS column.

$sth = $dbh->table_info();
$result = $sth->fetchall_arrayref({});
my @required = (qw(TABLE_CAT TABLE_SCHEM TABLE_NAME TABLE_TYPE));
my %missing;
for my $r (@$result) {
	for (@required) {
		$missing{$_}++ if ! exists $r->{$_};
	}
}
is_deeply( \%missing, {}, 'DB handle method "table_info" returns fields required by DBI');

# Check results

DBI->trace(3);
$sth = $dbh->table_info();
my $number = $sth->rows();
ok( $number, 'DB handle method "table_info" works when called with undef arguments');

$result = $sth->fetchall_hashref('TABLE_NAME');
DBI->trace(0);
is( $result->{'dbd_fb_test'}->{TABLE_CAT}, $catalog, 'DB handle method "table_info" returns proper TABLE_CAT');
is( $result->{'dbd_fb_test'}->{TABLE_SCHEM}, uc $user, 'DB handle method "table_info" returns proper TABLE_NAME');
is( $result->{'dbd_fb_test'}->{TABLE_TYPE}, 'BASE TABLE','DB handle method "table_info" returns proper TABLE_TYPE');
is( $result->{'dbd_fb_test'}->{REMARKS}, undef ,'DB handle method "table_info" returns proper REMARKS');

#
# Test of the "column_info" database handle method.
# Not supported but will be added.
#

if (!$dbh->can('column_info')) {
	ok(!defined $dbh->can('column_info'), 'column_info is not defined');
	diag 'Driver does not support "column_info"';
}
else {
	fail('column_info test');
}

#
# Test of the "primary_key_info" database handle method.
# Not supported.
#

if (!$dbh->can('primary_key_info')) {
	ok(!defined $dbh->can('primary_key_info'), 'primary_key_info is not defined');
	diag 'Driver does not support "primary_key_info"';
}
else {
	fail('primary_key_info test');
}

#
# Test of the "primary_key" database handle method.
# Not supported. Always returns an empty list.
#

@result = $dbh->primary_key('','','');
ok(scalar @result == 0, 'No result from primary_key');

#
# Test of the "foreign_key_info" database handle method.
# Not supported.
#

if (!$dbh->can('foreign_key_info')) {
	ok(!defined $dbh->can('foreign_key_info'), 'foreign_key_info is not defined');
	diag 'Driver does not support "foreign_key_info"';
}
else {
	fail('foreign_key_info test');
}

#
# Test of the "statistics_info" database handle method.
# Not supported.
#

if (!$dbh->can('statistics_info')) {
	ok(!defined $dbh->can('statistics_info'), 'statistics_info is not defined');
	diag 'Driver does not support "statistics_info"';
}
else {
	fail('statistics_info test');
}

#
# Test of the "tables" database handle method.
# This does not return views.
# This does not support any parameters.
# This only returns tables owned by the current user.
#

my (@names, $testname);
@names = $dbh->tables;
ok( @names, 'DB handle method "tables" works when called with undef arguments');

# This loop can be rewritten to use for(@names)
if (@names) {
	foreach my $name(@names) {
		if($name eq 'dbd_fb_test') {
			$testname = $name;
			last;
		}
	}
}
ok( $testname, 'DB handle method "table_info" returns the test table');

#
# Test of the "type_info_all" database handle method
# Type information is currently incomplete and out of date.
#

$result = $dbh->type_info_all;

# Quick check that the structure looks correct
my $badresult="";
if (ref $result eq "ARRAY") {
	my $index = $result->[0];
	if (ref $index ne "HASH") {
		$badresult = "First element in array not a hash ref";
	}
	else {
		for (qw(TYPE_NAME DATA_TYPE CASE_SENSITIVE)) {
			$badresult = "Field $_ missing" if !exists $index->{$_};
		}
	}
}
else {
	$badresult = "Array reference not returned";
}
diag "type_info_all problem: $badresult" if $badresult;
ok ( !$badresult, 'DB handle method "type_info_all" returns a valid structure');

#
# Test of the "type_info" database handle method
# This currently fails, the driver only returns the first 15 columns and uses the old names.
#

# Check required minimum fields
$result = $dbh->type_info(4);
@required = 
	(qw(TYPE_NAME DATA_TYPE COLUMN_SIZE LITERAL_PREFIX LITERAL_SUFFIX 
			CREATE_PARAMS NULLABLE CASE_SENSITIVE SEARCHABLE UNSIGNED_ATTRIBUTE 
			FIXED_PREC_SCALE AUTO_UNIQUE_VALUE LOCAL_TYPE_NAME MINIMUM_SCALE 
			MAXIMUM_SCALE SQL_DATA_TYPE SQL_DATETIME_SUB NUM_PREC_RADIX 
			INTERVAL_PRECISION));
undef %missing;
for (@required) {
	$missing{$_}++ if ! exists $result->{$_};
}
is_deeply( \%missing, {}, 'DB handle method "type_info" returns fields required by DBI');

#
# Test of the "quote" database handle method
#

my %quotetests = (
									q{0} => q{'0'},
									q{Ain't misbehaving } => q{'Ain''t misbehaving '},
									NULL => q{'NULL'},
									"" => q{''},
									q{\a} => q{'\\\\a'}
								 );

for (keys %quotetests) {
	$result = $dbh->quote($_);
	is( $result, $quotetests{$_}, qq{DB handle method "quote" works with a value of "$_"});
}

#
# Test of the "quote_identifier" database handle method
#

%quotetests = (
									q{0} => q{"0"},
									q{Ain't misbehaving } => q{"Ain't misbehaving "},
									NULL => q{"NULL"},
									"" => q{""}
							);
for (keys %quotetests) {
	$result = $dbh->quote_identifier($_);
	is( $result, $quotetests{$_}, qq{DB handle method "quote_identifier" works with a value of "$_"});
}
is( $dbh->quote_identifier(undef), q{}, 'DB handle method "quote_identifier" works with an undefined value');

is ($dbh->quote_identifier( undef, 'Her schema', 'My table' ), q{"Her schema"."My table"}, 
		'DB handle method "quote_identifier" works with schemas');
		
is ($dbh->quote_identifier( 'Test', 'Her schema', 'My table' ), q{"Test"."Her schema"."My table"}, 
		'DB handle method "quote_identifier" works with schemas');
		
		
$sth->finish;

ok($dbh->ping, 'Database is connected');

is($dbh->func('welcome', '_SYSTEM', "fb_digest_password"), '89f9a21fc1b06d90', 'fb_digest_password correct with both parms');
is($dbh->func('welcome', 'colin', "fb_digest_password"), '7923f20456b3c028', 'fb_digest_password correct with lower case user');
is($dbh->func('welcome', "fb_digest_password"), '89f9a21fc1b06d90', 'fb_digest_password correct with no user');
is($dbh->func('', '_SYSTEM', "fb_digest_password"), '', 'fb_digest_password correct with empty password');
ok(!$dbh->func("fb_digest_password"), 'fb_digest_password correct with no password');

# Example

#	my $passwd = 'welcome';
#	my $username = 'colin';
#	my $digest = $dbh->func($passwd, $username, "fb_digest_password");
#	$sth = $dbh->prepare('alter user "'.$username.'" set passwd \''.$digest.'\'');
#	$sth->execute();
#	$dbh->commit();

$dbh->disconnect;
is($dbh->func('welcome', "fb_digest_password"), '89f9a21fc1b06d90', 'fb_digest_password correct with no user and diconnected');
ok(!defined $dbh->ping, 'Database is not connected');