#!perl -w

# Test of placeholders

use Test::More tests => 20;
use DBI;
use Sys::Hostname;
use strict;
$|=1;

# defaults for connection

my $dsn = $ENV{DBI_DSN};
my $user = "_SYSTEM";
my $passwd = "";
my $hostname = hostname;
my $dbname = "test";
my $dbpasswd = "";

# DSN_xxx values override defaults

if($dsn && $dsn =~ s/^(dbi):(\w*?)(?:\((.*?)\))?://i && lc($2) eq 'fb') {
	$dbpasswd = $1 if($3 && $3 =~ /\bfb_dbpasswd\b\s*=>\s*(\w+)/);
	$dbname = $1 if($dsn =~ /\bdbname\b\s*=\s*(\w+)/);
	$hostname = $1 if($dsn =~ /\bhost\b\s*=\s*([\.A-Za-z0-9_]+)/);
}
$user = $ENV{DBI_USER} if ($ENV{DBI_USER});
$passwd = $ENV{DBI_PASS} if ($ENV{DBI_PASS});

# FB_xxx values override DSN_xxx values

$hostname = $ENV{FB_HOST} if ($ENV{FB_HOST});
$dbname = $ENV{FB_DBNAME} if ($ENV{FB_DBNAME});
$dbpasswd = $ENV{FB_DBPASSWD} if ($ENV{FB_DBPASSWD});
$user = $ENV{FB_USER} if ($ENV{FB_USER});
$passwd = $ENV{FB_PASSWD} if ($ENV{FB_PASSWD});

my $dbh = DBI->connect("dbi:FB:dbname=$dbname;host=$hostname", $user, $passwd,
							{fb_dbpasswd => $dbpasswd, RaiseError => 1, PrintError => 0, AutoCommit => 0});
							
ok( defined $dbh, 'Connect to database for placeholder testing');

# Make sure that quoting works properly.
my $quo = $dbh->quote("\\'?:");
is( $quo, "'\\\\''?:'", "Properly quoted");

# Make sure that quoting works with a function call.
# It has to be in this function, otherwise it doesn't fail the
# way described in https://rt.cpan.org/Ticket/Display.html?id=4996.
sub checkquote {
    my $str = shift;
    is( $dbh->quote(substr($str, 0, 10)), "'$str'", "First function quote");
}

checkquote('one');
checkquote('two');
checkquote('three');
checkquote('four');

my $sth = $dbh->prepare(qq{INSERT INTO dbd_fb_test (id,pname) VALUES (?, $quo)});
$sth->execute(100);

my $sql = "SELECT pname FROM dbd_fb_test WHERE pname = $quo";
$sth = $dbh->prepare($sql);
$sth->execute();

my ($retr) = $sth->fetchrow_array();
ok( (defined($retr) && $retr eq "\\\\'?:"), "fetch");

eval {
	$sth = $dbh->prepare($sql);
	$sth->execute('foo');
};
ok( $@, 'execute with one bind param where none expected');

$sql = "SELECT pname FROM dbd_fb_test WHERE pname = ?";
$sth = $dbh->prepare($sql);
$sth->execute("\\'?:");

($retr) = $sth->fetchrow_array();
ok( (defined($retr) && $retr eq "\\\\'?:"), 'execute with ? placeholder');

$sql = "SELECT pname FROM dbd_fb_test WHERE pname = :1";
$sth = $dbh->prepare($sql);
# Note. can't use bind_param(":1",....)
$sth->bind_param(":p1", "\\'?:");
$sth->execute();

($retr) = $sth->fetchrow_array();
ok( (defined($retr) && $retr eq "\\\\'?:"), 'execute with :1 placeholder');

$sql = q{SELECT pname FROM dbd_fb_test WHERE pname = :p1 AND pname <> 'foo'};
$sth = $dbh->prepare($sql);
$sth->execute("\\'?:");

($retr) = $sth->fetchrow_array();
ok( (defined($retr) && $retr eq "\\\\'?:"), 'execute with :p1 placeholder');

$sql = "SELECT pname FROM dbd_fb_test WHERE pname = '?'";

eval {
	$sth = $dbh->prepare($sql);
	$sth->execute('foo');
};
ok( $@, 'execute with quoted ?');

$sql = "SELECT pname FROM dbd_fb_test WHERE pname = ':1'";

eval {
	$sth = $dbh->prepare($sql);
	$sth->execute('foo');
};
ok( $@, 'execute with quoted :1');

$sql = "SELECT pname FROM dbd_fb_test WHERE pname = '\\\\' AND pname = '?'";
$sth = $dbh->prepare($sql);

eval {
## XX ???
	local $dbh->{PrintError} = 0;
	local $sth->{PrintError} = 0;
	$sth->execute('foo');
};
ok( $@, 'execute with quoted ?');

## Test large number of placeholders
$sql = 'SELECT 1 FROM dbd_fb_test WHERE id IN (' . '?,' x 300 . "?)";
my @args = map { $_ } (1..301);
$sth = $dbh->prepare($sql);
my $count = $sth->execute(@args);
$sth->finish();
ok( $count >= 1, 'prepare with large number of parameters works');

$sth->finish();

## Test our parsing of backslashes
# Note. The driver only auto-escapes parameter values
# It thinks there is a placeholder in here
$sth = $dbh->prepare("VALUES '\\''?'");
eval {
	$sth->execute();
};
ok( !$@, 'prepare with backslashes inside quotes works');
$sth->finish();
$dbh->rollback;

## Test do() with placeholders, both DML and non-DML
eval {
  $dbh->do(q{SET SCHEMA ?}, undef, '_SYSTEM');
};
ok( !$@, 'do() called with non-DML placeholder works');
eval {
  $dbh->do(q{SELECT ? FROM dbd_fb_test}, undef, 'public');
};
ok( !$@, 'do() called with DML placeholder works');
$dbh->rollback;
## Test a non-DML placeholder
eval {
  $sth = $dbh->prepare(qq{SET SCHEMA ?});
  $sth->execute('_SYSTEM');
};
ok( !$@, 'prepare/execute with non-DML placeholder works');

$dbh->rollback();

ok( $dbh->disconnect(), 'Disconnect from database');

