#!perl -w

# Test passing FB datatypes

use Test::More tests => 17;
use DBI;
use Sys::Hostname;
use strict;
use DBI qw(:sql_types);
$|=1;

# defaults for connection

my $dsn = $ENV{DBI_DSN};
my $user = "_SYSTEM";
my $passwd = "";
my $hostname = hostname;
my $dbname = "test";
my $dbpasswd = "";

# DSN_xxx values override defaults

if($dsn && $dsn =~ s/^(dbi):(\w*?)(?:\((.*?)\))?://i && lc($2) eq 'fb') {
	$dbpasswd = $1 if($3 && $3 =~ /\bfb_dbpasswd\b\s*=>\s*(\w+)/);
	$dbname = $1 if($dsn =~ /\bdbname\b\s*=\s*(\w+)/);
	$hostname = $1 if($dsn =~ /\bhost\b\s*=\s*([\.A-Za-z0-9_]+)/);
}
$user = $ENV{DBI_USER} if ($ENV{DBI_USER});
$passwd = $ENV{DBI_PASS} if ($ENV{DBI_PASS});

# FB_xxx values override DSN_xxx values

$hostname = $ENV{FB_HOST} if ($ENV{FB_HOST});
$dbname = $ENV{FB_DBNAME} if ($ENV{FB_DBNAME});
$dbpasswd = $ENV{FB_DBPASSWD} if ($ENV{FB_DBPASSWD});
$user = $ENV{FB_USER} if ($ENV{FB_USER});
$passwd = $ENV{FB_PASSWD} if ($ENV{FB_PASSWD});

my $dbh = DBI->connect("dbi:FB:dbname=$dbname;host=$hostname", $user, $passwd,
							{fb_dbpasswd => $dbpasswd, RaiseError => 1, PrintError => 0});

ok( defined $dbh, 'Connect to database for datatype testing');

my ($sth, $SQL, $result, $expected);

#
# Insert with DATE, TIME and TIMESTAMP bound as strings
#

my $false = 'False';
my $true = 'True';

$SQL = qq {
	INSERT INTO dbd_fb_datatypes ( 
		char12_, 
		varchar20_, 
		date_,
		time_,
		timestamp_,
		timewithzone_,
		timestampwithzone_,
		int4_,
		decimal_,
		clob_,
		blob_,
		boolean_
	)
	VALUES ( ?, ?, DATE ?, TIME ?, TIMESTAMP ?, TIME ?, TIMESTAMP ?, ?, ?, ?, ?, ?)
};

eval {
	 $sth = $dbh->prepare($SQL);
};

ok(!$@, 'First insert prepared');

$sth->bind_param(10, undef, SQL_LONGVARCHAR);
$sth->bind_param(11, undef, SQL_LONGVARBINARY);
$sth->bind_param(12, undef, SQL_BOOLEAN);

$result = $sth->execute(
	'1',
	'Roj Blake',
	'1997-12-01',
	'12:51:10',
	'1997-12-01 12:51:10',
	'12:51:10+01:00',
	'1997-12-01 12:51:10+01:00',
	1234,
	1.1,
	'Row1 CLOB data',
	'Row1 BLOB data',
	1
);

is($result, 1, 'First insert executed');

#
# Insert with DATE, TIME and TIMESTAMP bound by type
#

$SQL = qq {
	INSERT INTO dbd_fb_datatypes ( 
		char12_, 
		varchar20_, 
		date_,
		time_,
		timestamp_,
		timewithzone_,
		timestampwithzone_,
		int4_,
		decimal_,
		clob_,
		blob_,
		boolean_
	)
	VALUES ( ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
};

eval {
	 $sth = $dbh->prepare($SQL);
};

ok(!$@, 'Second insert prepared');

$sth->bind_param(3, undef, SQL_DATE);
$sth->bind_param(4, undef, SQL_TIME);
$sth->bind_param(5, undef, SQL_TIMESTAMP);
$sth->bind_param(6, undef, SQL_TIME);
$sth->bind_param(7, undef, SQL_TIMESTAMP);
$sth->bind_param(10, undef, SQL_LONGVARCHAR);
$sth->bind_param(11, undef, SQL_LONGVARBINARY);
$sth->bind_param(12, undef, SQL_BOOLEAN);

$result = $sth->execute(
	'2',
	'John Lovejoy',
	'1997-12-02',
	'12:52:10',
	'1997-12-02 12:52:10',
	'12:52:10+01:00',
	'1997-12-02 12:52:10+01:00',
	2345,
	2.2,
	'Row2 CLOB data',
	'Row2 BLOB data',
	'0'
);

is($result, 1, 'Second insert executed');

eval {
	$sth = $dbh->prepare("SELECT * from dbd_fb_datatypes");
};

ok(!$@, 'SELECT prepared');

$result = $sth->execute();
is($result, 2, 'SELECT executed with correct rows returned');

$expected = ['1', 'Roj Blake', '1997-12-01', '+12:51:10', 
			'1997-12-01 12:51:10.000000', '+12:51:10+01:00', 
			'1997-12-01 12:51:10.000000+01:00', 1234, '1.100000', 
			'Row1 CLOB data', 'Row1 BLOB data', 'TRUE'];

$result = $sth->fetch();
is_deeply( $result, $expected, 'First row fetched correctly');

$expected = ['2', 'John Lovejoy', '1997-12-02', '+12:52:10', 
			'1997-12-02 12:52:10.000000', '+12:52:10+01:00', 
			'1997-12-02 12:52:10.000000+01:00', 2345, '2.200000', 
			'Row2 CLOB data', 'Row2 BLOB data', 'FALSE'];
			
$result = $sth->fetch();
is_deeply( $result, $expected, 'Second row fetched correctly');

#
# Test of binding FB_SQLIdentifier
#

my $newuser = '"sparky"';

eval {
	$sth = $dbh->prepare('DROP USER ? cascade');
};

ok(!$@, 'DROP USER prepared');
ok($sth->bind_param(1, $newuser, {fb_type => 99}), 'Bind DROP USER param');

eval {
	$sth->execute;
};

eval {
	$sth = $dbh->prepare('CREATE USER ?');
};

ok(!$@, 'CREATE USER prepared');
ok($sth->bind_param(1, $newuser, {fb_type => 99}), 'Bind CREATE USER param');
ok($sth->execute, 'Execute CREATE USER');
	
eval {
	$sth = $dbh->prepare('DROP USER ? restrict')
};

ok(!$@, 'DROP USER prepared');
ok($sth->bind_param(1, $newuser, {fb_type => 99}), 'Bind DROP USER param');		
ok($sth->execute, 'Execute DROP USER');

$sth->finish;
$dbh->disconnect();